/***************************************************************************
                          performanceitemreader.cpp  -  description
                             -------------------
    begin                : Sat Jun 9 2001
    copyright            : (C) 2001 by Roberto Virga
    email                : rvirga@users.sourceforge.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <klocale.h>

#include "performanceitemreader.h"

PerformanceItemReader::PerformanceItemReader(QListView *list) : list(list)
{
}

PerformanceItemReader::~PerformanceItemReader()
{
}

bool PerformanceItemReader::startElement(const QString &, const QString &,
                                         const QString &qName, const QXmlAttributes &attributes)
{
	if(qName == "entry")
	{
		// initialize data
		data.name = data.cache = QString::null;
		data.min = data.max = 1.0;
		for(int i = 0; i < N_COEFF; i++)
			data.coeff[i] = 0.0;
	}
	else if(qName == "name")
	{
		// read # of cpus and type of memory
		cpus = parseCPUs(attributes.value("cpus"));
		memory = attributes.value("memory");
	}
	else if(qName == "cache")
	{
		// read cache level
		QString levelStr = attributes.value("level");
		level = levelStr.isEmpty() ? 2 : levelStr.toInt();
		// read size unit
		QString unitStr = attributes.value("unit");
		unit = unitStr.isEmpty() ? QString("KB") : unitStr.upper();
	}
	else if(qName == "coeff")
	{
		// read degree of the polynomial coefficient
		QString degStr = attributes.value("deg");
		deg = degStr.isEmpty() ? 0 : degStr.toInt();
	}
	return(true);
}

bool PerformanceItemReader::endElement(const QString &, const QString &, const QString &qName)
{
	if(qName == "entry")
	{
		data.min = (data.min > data.max) ? data.max : data.min;
		new PerformanceItem(list, data);
	}
	else if(qName == "name")
	{
		data.name = cdata;
		if(!cpus.isEmpty() && !memory.isEmpty())
			data.name += (" (" + cpus + " - " + memory + ")");
		else if(!cpus.isEmpty())
			data.name += (" (" + cpus + ")");
		else if(!memory.isEmpty())
			data.name +=(" (" + memory + ")");
	}
	else if(qName == "cache")
	{
		if(level == 2)
			data.cache = cdata + " " + unit;
	}
	else if(qName == "min")
	{
		data.min = cdata.toDouble();
		data.min = (data.min < 1.0 || data.min > 20.0) ? 1.0 : data.min;
	}
	else if(qName == "max")
	{
		data.max = cdata.toDouble();
		data.max = (data.max < 1.0 || data.max > 20.0) ? 1.0 : data.max;
	}
	else if(qName == "coeff")
	{
		bool parseOK;
		double coeffDouble = cdata.toDouble(&parseOK);
		if(parseOK && deg >= 0 && deg < N_COEFF)
			data.coeff[deg] = coeffDouble;
	}
	return(true);
}

bool PerformanceItemReader::characters(const QString &cdata)
{
	this->cdata = cdata;
	return(true);
}

QString PerformanceItemReader::parseCPUs(const QString &cpusStr)
{
	bool parseOK;
	int cpusInt = cpusStr.toInt(&parseOK);

	cpusInt = (!parseOK || cpusInt < 1) ? 1 : cpusInt;

	switch(cpusInt) {
		case 1:
			return(QString::null);
		case 2:
			return(i18n("Dual"));
		case 4:
			return(i18n("Quad"));
		default:
			return(QString::number(cpusInt) + " " + i18n("CPUs"));
	}
	return(QString::null);
}
